#include "std.h"

#include <doublefann.h>

// *** CREATION / EXECUTION / IMPORT / EXPORT ***

MT4_EXPFUNC int __stdcall FANN_Create( int num_input, int num_output, int num_neurons_hidden )
{
  fann * handle = fann_create_standard( 3, num_input, num_neurons_hidden, num_output );

	fann_set_activation_steepness_hidden(handle, 1);
	fann_set_activation_steepness_output(handle, 1);

	fann_set_activation_function_hidden(handle, FANN_SIGMOID_SYMMETRIC);
	fann_set_activation_function_output(handle, FANN_SIGMOID_SYMMETRIC);

	fann_set_train_stop_function(handle, FANN_STOPFUNC_BIT);
	fann_set_bit_fail_limit(handle, 0.01f);

  return (int)handle;
}

MT4_EXPFUNC int __stdcall FANN_CreateFromFile( char * fname )
{
  fann * handle = fann_create_from_file( fname );
  return (int)handle;
}

MT4_EXPFUNC void __stdcall FANN_Save( int handle, char * fname )
{
  fann * f = (fann*) handle;
  fann_save( f, fname );  
}

MT4_EXPFUNC void __stdcall FANN_Destroy( int handle )
{
  fann * f = (fann*) handle;
  fann_destroy( f );
}

// *** TRAIN/TEST/RUN ***

#define max_input 1024
#define max_output 256

/*
void convert_fann_to_double_array( int n, const fann_type * in, double * out )
{
  for ( int i = 0; i < n; i++ )
    out[i] = (double) in[i];
}

void convert_double_to_fann_array( int n, const double * in, fann_type * out )
{
  for ( int i = 0; i < n; i++ )
    out[i] = (fann_type) in[i];
};
*/

MT4_EXPFUNC void __stdcall FANN_Train( int handle, const double * input, const double * desired_output )
{
  // convert parameters
  fann * f = (fann*) handle;
  fann_train( f, (fann_type *)input, (fann_type*) desired_output );
}

MT4_EXPFUNC double __stdcall FANN_Test( int handle, const double * input, const double * desired_output )
{
  // convert parameters
  fann * f = (fann*) handle;

  fann_test( f, (fann_type *)input, (fann_type*) desired_output );

  return fann_get_MSE( f );
}

MT4_EXPFUNC void __stdcall FANN_Run( int handle, const double * in, double * out )
{
  // convert parameters
  fann * f = (fann*) handle;

  fann_type * fann_output = fann_run( f, (fann_type*)in );
  memcpy( out, fann_output, sizeof(double) * f->num_output );
}

void fann_test()
{
  const unsigned int num_input = 512;
  const unsigned int num_output = 64;
  const unsigned int num_layers = 3;
  const unsigned int num_neurons_hidden = 3;
  const float desired_error = (const float) 0.01;
  const unsigned int max_epochs = 500000;
  const unsigned int epochs_between_reports = 1000;

  struct fann *ann = fann_create_standard(num_layers, num_input, num_neurons_hidden, num_output);

  fann_set_activation_function_hidden(ann, FANN_SIGMOID_SYMMETRIC);
  fann_set_activation_function_output(ann, FANN_SIGMOID_SYMMETRIC);

  fann_train_on_file(ann, "xor.data", max_epochs, epochs_between_reports, desired_error);

  fann_save(ann, "xor_float.net");

  fann_destroy(ann);
}
